/*
 * (c) 2007-2008, M-logic, Inc. All rights reserved.
 *
 * $Id$
 */

MT.App.FullscreenMode = new Class(Object, {
    init: function() {
        this.enabled = false;
        this.elements = {};
        this.elements.html = document.getElementsByTagName("html")[0];
        this.elements.template_body = DOM.getElement("template-body-field");
        this.elements.field_content = DOM.getElementsByClassName(this.elements.template_body, "field-content")[0];
        this.elements.toolbar = DOM.getElement("template-body-actions");
        this.elements.enclosure = DOM.getElement("textarea-enclosure");
        this.elements.wrapper = DOM.getElementsByTagAndClassName(this.elements.enclosure, "div", "textarea-wrapper")[0];
        this.elements.text = DOM.getElement("text");
        this.elements.text_cpe = DOM.getElement("text_cpe");
        this.elements.resizer = DOM.getElementsByTagAndClassName(this.elements.field_content, "div", "resizer")[0];
        this.elements.template_options = DOM.getElement("template-options");
        this.styleInfo = {};
        this.styleInfo.html = {};
    },
    destroy: function() {
        this.elements = null;
        this.styleInfo.html = null;
        this.styleInfo = null;
    },
    toggleFullscreenMode: function() {
        this.togglePulldownMenu();
        if(!this.enabled) {
            // hide resizer
            DOM.addClassName(this.elements.resizer, "hidden");
            // hide template options
            if(this.elements.template_options) {
                DOM.addClassName(this.elements.template_options, "hidden");
            }
            // set html info
            this.saveHtmlStyleInfo();
            this.elements.html.style.overflow = "hidden";
            this.elements.html.scrollTop = 0;
            this.elements.html.scrollLeft = 0;
            var screen = DOM.getClientDimensions();
            // set content area info
            this.saveStyleInfo(this.styleInfo, this.elements.field_content, "field-content");
            this.elements.field_content.style.position = "absolute";
            this.elements.field_content.style.width = screen.x - this.getEdgeWidth(this.elements.field_content) + "px";
            this.elements.field_content.style.height = screen.y - this.getEdgeHeight(this.elements.field_content) + "px";
            this.elements.field_content.style.display = "block";
            this.elements.field_content.style.zIndex = "999999";
            this.elements.field_content.style.top = "0px";
            this.elements.field_content.style.left = "0px";
            // resize textarea and iframe
            this.resizeTextarea(true);
            this.enabled = true;
        }
        else {
            // restore html info
            this.restoreHtmlStyleInfo();
            // restore content area
            this.restoreStyleInfo(this.styleInfo, this.elements.field_content, "field-content");
            this.restoreStyleInfo(this.styleInfo, this.elements.enclosure, "text-enclosure");
            // resize textarea and iframe
            this.restoreStyleInfo(this.styleInfo, this.elements.text, "text");
            this.restoreStyleInfo(this.styleInfo, this.elements.text_cpe, "text_cpe");
            // restore editor state
            this.elements.text.style.display = this.elements.text_cpe.disabled ? 'inline' : 'none';
            this.elements.text_cpe.style.display = this.elements.text_cpe.disabled ? 'none' : 'inline';
            // show resizer
            DOM.removeClassName(this.elements.resizer, "hidden");
            // show template options
            // HACK: IE6 & MT4.2, to redraw id=template-options at the correct position, reset the position attribute;
            if(this.elements.template_options) {
                DOM.removeClassName(this.elements.template_options, "hidden");
            }
            this.enabled = false;
        }
    },
    getEdgeWidth: function(e) {
        var w;
        if(e) {
            var s = DOM.getComputedStyle(e);
            w = finiteInt(s.borderLeftWidth) + finiteInt(s.paddingLeft)
                + finiteInt(s.paddingRight) + finiteInt(s.borderRightWidth);
        }
        return w;
    },
    getEdgeHeight: function(e) {
        var h;
        if(e) {
            var s = DOM.getComputedStyle(e);
            h = finiteInt(s.borderTopWidth) + finiteInt(s.paddingTop)
                + finiteInt(s.paddingBottom) + finiteInt(s.borderBottomWidth);
        }
        return h;
    },
    saveHtmlStyleInfo: function() {
        this.styleInfo.html.overflow = this.elements.html.style.overflow;
        this.styleInfo.html.scrollTop = this.elements.html.scrollTop;
        this.styleInfo.html.scrollLeft = this.elements.html.scrollLeft;
    },
    restoreHtmlStyleInfo: function() {
        this.elements.html.style.overflow = this.styleInfo.html.overflow;
        this.elements.html.scrollTop = this.styleInfo.html.scrollTop;
        this.elements.html.scrollLeft = this.styleInfo.html.scrollLeft;
    },
    saveStyleInfo: function(storage, e, name) {
        if(!e || !name) return false;
        if(typeof(storage[name]) == "undefined") {
            storage[name] = {};
        }
        storage[name].position = e.style.position;
        storage[name].width = e.style.width;
        storage[name].height = e.style.height;
        storage[name].display = e.style.display;
        storage[name].zIndex = e.style.zIndex;
        storage[name].top = e.style.top;
        storage[name].left = e.style.left;
        storage[name].overflow = e.style.overflow;
        return true;
    },
    restoreStyleInfo: function(storage, e, name) {
        if(!e || !name) return false;
        if(typeof(storage[name]) == "undefined") {
            return false;
        }
        e.style.position = storage[name].position;
        e.style.width = storage[name].width;
        e.style.height = storage[name].height;
        e.style.display = storage[name].display;
        e.style.zIndex = storage[name].zIndex;
        e.style.top = storage[name].top;
        e.style.left = storage[name].left;
        e.style.overflow = storage[name].overflow;
        return true;
    },
    togglePulldownMenu: function() {
        // to avoid IE's bug(?)
        if(!document.all) {
            return;
        }
        var es = document.getElementsByTagName("select");
        for(var i=0; i<es.length; i++) {
            if(es[i].id != "tag-insert") {
                if(DOM.hasClassName(es[i], "hidden")) {
                    DOM.removeClassName(es[i], "hidden");
                }
                else {
                    DOM.addClassName(es[i], "hidden");
                }
            }
        }
    },
    resizeTextarea: function(save) {
        var screen = DOM.getClientDimensions();
        // set content area info
        this.elements.field_content.style.width = screen.x - this.getEdgeWidth(this.elements.field_content) + "px";
        this.elements.field_content.style.height = screen.y - this.getEdgeHeight(this.elements.field_content) + "px";
        // resize textarea and iframe
        if(save) {
            this.saveStyleInfo(this.styleInfo, this.elements.enclosure, "textarea-enclosure");
        }
        var width, height;
        if(this.elements.wrapper) {
            width = this.elements.wrapper.offsetWidth - this.getEdgeWidth(this.elements.wrapper);
            height = this.elements.field_content.offsetHeight - this.elements.toolbar.offsetHeight - this.getEdgeHeight(this.elements.wrapper);
        }
        else {
            width = this.elements.field_content.offsetWidth;
            height = this.elements.field_content.offsetHeight - this.elements.toolbar.offsetHeight;
        }
        var es = [ "text", "text_cpe" ];
        for(var i=0; i<es.length; i++) {
            var e = this.elements[es[i]];
            if(save) {
                this.saveStyleInfo(this.styleInfo, e, es[i]);
            }
            DOM.setHeight(e, height - this.getEdgeHeight(e));
            DOM.setWidth(e, width - this.getEdgeWidth(e));
        }
    }
});

MT.App.CodePress = new Class(MT.App.CodePress, {
    setLineNumberBackground: function(lineHeight) {
        if(lineHeight == 10 || lineHeight == 12) {
            var file = StaticURI+"plugins/FullScreenEdit/images/"+"line-numbers-"+lineHeight+".png";
            if(!DOM.hasClassName(this.editor, "hide-line-numbers")) {
                this.editor.body.style.backgroundImage = "url("+file+")";
                this.editor.body.style.marginTop = (lineHeight-1) + "px";
                this.editor.body.style.backgroundPosition = "0pt 0px";
            }
        }
        else {
            this.editor.body.style.backgroundImage = "";
            this.editor.body.style.marginTop = "";
            this.editor.body.style.backgroundPosition = "";
        }
    },
    changeFontSize: function(increase) {
        // textarea font size
        var ratio = increase ? 1.2 : 0.83;
        var cur_size = finiteInt(DOM.getStyle(this.textarea, "fontSize"));
        var new_size = Math.round(cur_size * ratio);
        if(new_size > 0 && ((!increase && new_size >= 9) || (increase && new_size <= 16))) {
            this.textarea.style.fontSize = new_size + "px";
        }
        // iframe font size
        var sheets = this.iframe.contentWindow.document.styleSheets;
        for(var i=0; i<sheets.length; i++) {
            if(sheets[i].href.indexOf("codepress.css") >= 0) {
                var rules = sheets[i].cssRules ? sheets[i].cssRules : sheets[i].rules;
                for(var n=0; n<rules.length; n++) {
                    var selectorText = rules[n].selectorText.toLowerCase();
                    var reg = new RegExp("^body$|^b$|^i|^s|^u|^a|^em|^tt|^ins|^big|^cite|^strong|^var|^dfn|^b, i, s, u, a, em, tt, ins, big, cite, strong, var, dfn$", "i");
                    if(reg.exec(selectorText) != null) {
                        cur_size = finiteInt(rules[n].style.fontSize);
                        new_size = Math.round(cur_size * ratio);
                        if(new_size > 0 && ((!increase && new_size >= 9) || (increase && new_size <= 16))) {
                            rules[n].style.fontSize = new_size + "px";
                            var lineHeight = 16;
                            if(new_size == 9) {
                                lineHeight = 10;
                            }
                            else if(new_size == 11) {
                                lineHeight = 12;
                            }
                            rules[n].style.lineHeight = lineHeight + "px";
                            this.setLineNumberBackground(lineHeight);
                        }
                    }
                }
                return;
            }
        }
    }
});

App.singletonConstructor =
MT.App = new Class(MT.App, {
    initComponents: function() {
        arguments.callee.applySuper(this, arguments);
        if(this.constructor.FullscreenMode) {
            this.fullscreenMode = new this.constructor.FullscreenMode();
        }
    },
    destroyObject: function() {
        this.fullscreenMode = null;
        arguments.callee.applySuper(this, arguments);
    },
    eventClick: function(event) {
        var command = this.getMouseEventCommand(event);
        switch(command) {
        case "fullscreenMode":
            this.fullscreenMode.toggleFullscreenMode();
            break;
        case "fontSizeSmaller":
            this.cpeList.forEach( function( cpe ) { cpe.changeFontSize(false); } );
            break;
        case "fontSizeLarger":
            this.cpeList.forEach( function( cpe ) { cpe.changeFontSize(true); } );
            break;
        default:
            return arguments.callee.applySuper(this, arguments);
        }
        return event.stop();
    },
    eventResize: function(event) {
        if(this.fullscreenMode.enabled) {
            this.fullscreenMode.resizeTextarea(false);
        }
    }
});
